searchData={"content_type":"text/markdown","items":[{"doc":"`StdResult` provides a way to standardize the return types of Elixir functions.\n\nTo help with pattern matching, the API also provides 3 macros:\n\n- `ok!/1`: same as `ok/1` but can be used in guards, pattern matching, ...\n- `err!/1`: same as `err/1` but can be used in guards, pattern matching, ...\n- `unit!/0`: same as `unit/0` but can be used in guards, pattern matching, ...","ref":"StdResult.html","title":"StdResult","type":"module"},{"doc":"In this library, we will use the term `Ok` for `:ok tuple` / `:ok Result` and\n`Err` for `:error tuple` / `:error Result`.","ref":"StdResult.html#module-note","title":"Note - StdResult","type":"module"},{"doc":"iex> import StdResult\n    iex> System.fetch_env(\"PORT\")\n    ...> |> normalize_result() # This will transform `:error` into a `:error` tuple\n    ...> |> or_result(err(\"PORT env required\")) # If there is an error, explicit the message\n    ...> |> map(&String.to_integer/1) # If no error, parse the string as an integer\n    ...> |> and_then(&(if &1 >= 0, do: ok(&1), else: err(\"PORT must be a positive number, got: #{&1}\"))) # Test if number is positive\n\n    # The result will be either:\n    # - `{:ok, port}`\n    # - `{:error, \"PORT env required\"}`\n    # - `{:error, \"PORT must be a positive number, got:  \"}`","ref":"StdResult.html#module-examples","title":"Examples - StdResult","type":"module"},{"doc":"Returns the right operand if the result is `Ok`, otherwise returns the\n`Err` value.\n\nArguments passed to `and_result/2` are eagerly evaluated; if you are\npassing the result of a function call, it is recommended to use\n`and_then/2`, which is lazily evaluated.","ref":"StdResult.html#and_result/2","title":"StdResult.and_result/2","type":"function"},{"doc":"iex> StdResult.and_result({:ok, 2}, {:error, \"late error\"})\n    {:error, \"late error\"}\n\n    iex> StdResult.and_result({:error, \"early error\"}, {:error, \"foo\"})\n    {:error, \"early error\"}\n\n    iex> StdResult.and_result({:error, \"not a 2\"}, {:error, \"late error\"})\n    {:error, \"not a 2\"}\n\n    iex> StdResult.and_result({:ok, 2}, {:ok, \"different result type\"})\n    {:ok, \"different result type\"}","ref":"StdResult.html#and_result/2-examples","title":"Examples - StdResult.and_result/2","type":"function"},{"doc":"Calls the callback if the result is `Ok`, otherwise returns the `Err` value.\n\nThis function can be used for control flow based on Result values.","ref":"StdResult.html#and_then/2","title":"StdResult.and_then/2","type":"function"},{"doc":"iex> StdResult.and_then({:ok, 2}, &{:ok, &1 * 2})\n    {:ok, 4}\n\n    iex> StdResult.and_then({:ok, 1_000_000}, fn _ -> {:error, \"overflowed\"} end)\n    {:error, \"overflowed\"}\n\n    iex> StdResult.and_then({:error, \"not a number\"}, &(&1 * 2))\n    {:error, \"not a number\"}","ref":"StdResult.html#and_then/2-examples","title":"Examples - StdResult.and_then/2","type":"function"},{"doc":"Wrap any term inside a `:error` tuple.","ref":"StdResult.html#err/1","title":"StdResult.err/1","type":"function"},{"doc":"iex> StdResult.err(\"unexpected error\")\n    {:error, \"unexpected error\"}","ref":"StdResult.html#err/1-examples","title":"Examples - StdResult.err/1","type":"function"},{"doc":"Same as `err/1` but can be used in guards, pattern matching, ...","ref":"StdResult.html#err!/1","title":"StdResult.err!/1","type":"macro"},{"doc":"Returns `true` if the result is `Err`.","ref":"StdResult.html#err?/1","title":"StdResult.err?/1","type":"function"},{"doc":"iex> StdResult.err?({:ok, -3})\n    false\n\n    iex> StdResult.err?({:error, \"Some error message\"})\n    true","ref":"StdResult.html#err?/1-examples","title":"Examples - StdResult.err?/1","type":"function"},{"doc":"Returns `true` if the result is `Err` and the value inside of it matches\na predicate.","ref":"StdResult.html#err_and?/2","title":"StdResult.err_and?/2","type":"function"},{"doc":"iex> StdResult.err_and?({:error, :not_found}, &(&1 == :not_found))\n    true\n\n    iex> StdResult.err_and?({:error, :failed}, &(&1 == :not_found))\n    false\n\n    iex> StdResult.err_and?({:ok, 123}, &(&1 == :not_found))\n    false","ref":"StdResult.html#err_and?/2-examples","title":"Examples - StdResult.err_and?/2","type":"function"},{"doc":"Returns the contained `Ok` value.\n\nBecause this function may raise, its use is generally discouraged.\nInstead, prefer to use pattern matching and handle the `Err` case\nexplicitly, or call `unwrap_or/2` or `unwrap_or_else/2`.","ref":"StdResult.html#expect/2","title":"StdResult.expect/2","type":"function"},{"doc":"iex> StdResult.expect({:ok, 21}, \"Testing expect\")\n    21\n\n    iex> StdResult.expect({:error, \"emergency failure\"}, \"Testing expect\")\n    ** (RuntimeError) Testing expect: emergency failure","ref":"StdResult.html#expect/2-examples","title":"Examples - StdResult.expect/2","type":"function"},{"doc":"We recommend that `expect/2` messages are used to describe the reason you\nexpect the Result should be `Ok`.\n\n    iex> System.fetch_env(\"IMPORTANT_PATH\")\n    ...> |> StdResult.normalize_result()\n    ...> |> StdResult.expect(\"env variable `IMPORTANT_PATH` should be set by `wrapper_script.sh`\")\n    ** (RuntimeError) env variable `IMPORTANT_PATH` should be set by `wrapper_script.sh`: {}\n\n**Hint**: If you’re having trouble remembering how to phrase expect error\nmessages remember to focus on the word “should” as in “env variable should\nbe set by blah” or “the given binary should be available and executable\nby the current user”.","ref":"StdResult.html#expect/2-recommended-message-style","title":"Recommended Message Style - StdResult.expect/2","type":"function"},{"doc":"Returns the contained `Err` value.\n\nFor more details, see: `expect/2`","ref":"StdResult.html#expect_err/2","title":"StdResult.expect_err/2","type":"function"},{"doc":"iex> StdResult.expect_err({:error, \"emergency failure\"}, \"Testing expect\")\n    \"emergency failure\"\n\n    iex> StdResult.expect_err({:ok, 42}, \"Testing expect\")\n    ** (RuntimeError) Testing expect: 42","ref":"StdResult.html#expect_err/2-examples","title":"Examples - StdResult.expect_err/2","type":"function"},{"doc":"Calls the provided closure with a reference to the contained value (if `Ok`).","ref":"StdResult.html#inspect/2","title":"StdResult.inspect/2","type":"function"},{"doc":"iex> import ExUnit.CaptureIO\n    iex> capture_io(fn -> StdResult.inspect({:ok, 21}, &IO.inspect/1) end)\n    \"21\\n\"\n\n    iex> StdResult.inspect({:ok, 21}, &Function.identity/1)\n    {:ok, 21}\n\n    iex> import ExUnit.CaptureIO\n    iex> capture_io(fn -> StdResult.inspect({:error, 42}, &IO.inspect/1) end)\n    \"\"\n\n    iex> StdResult.inspect({:error, 42}, &Function.identity/1)\n    {:error, 42}","ref":"StdResult.html#inspect/2-examples","title":"Examples - StdResult.inspect/2","type":"function"},{"doc":"Calls the provided closure with a reference to the contained value (if `Err`).","ref":"StdResult.html#inspect_err/2","title":"StdResult.inspect_err/2","type":"function"},{"doc":"iex> import ExUnit.CaptureIO\n    iex> capture_io(fn -> StdResult.inspect_err({:ok, 21}, &IO.inspect/1) end)\n    \"\"\n\n    iex> StdResult.inspect_err({:ok, 21}, &Function.identity/1)\n    {:ok, 21}\n\n    iex> import ExUnit.CaptureIO\n    iex> capture_io(fn -> StdResult.inspect_err({:error, 42}, &IO.inspect/1) end)\n    \"42\\n\"\n\n    iex> StdResult.inspect_err({:error, 42}, &Function.identity/1)\n    {:error, 42}","ref":"StdResult.html#inspect_err/2-examples","title":"Examples - StdResult.inspect_err/2","type":"function"},{"doc":"Maps a Result into another by applying a function to a contained `Ok`\nvalue, leaving an `Err` value untouched.\n\nThis function can be used to compose the results of two functions.","ref":"StdResult.html#map/2","title":"StdResult.map/2","type":"function"},{"doc":"iex> StdResult.map({:ok, \"123\"}, &String.to_integer/1)\n    {:ok, 123}\n\n    iex> StdResult.map({:error, :not_found}, &String.to_integer/1)\n    {:error, :not_found}","ref":"StdResult.html#map/2-examples","title":"Examples - StdResult.map/2","type":"function"},{"doc":"Maps a Result into another by applying a function to a contained `Err`\nvalue, leaving an `Ok` value untouched.\n\nThis function can be used to pass through a successful result while\nhandling an error.","ref":"StdResult.html#map_err/2","title":"StdResult.map_err/2","type":"function"},{"doc":"iex> StdResult.map_err({:ok, 2}, &\"error code: #{&1}\")\n    {:ok, 2}\n\n    iex> StdResult.map_err({:error, 13}, &\"error code: #{&1}\")\n    {:error, \"error code: 13\"}","ref":"StdResult.html#map_err/2-examples","title":"Examples - StdResult.map_err/2","type":"function"},{"doc":"Returns the provided default (if `Err`), or applies a function to the\ncontained value (if `Ok`).\n\nArguments passed to `map_or/3` are eagerly evaluated; if you are passing\nthe result of a function call, it is recommended to use `map_or_else/3`,\nwhich is lazily evaluated.","ref":"StdResult.html#map_or/3","title":"StdResult.map_or/3","type":"function"},{"doc":"iex> StdResult.map_or({:ok, \"foo\"}, 42, &String.length/1)\n    3\n\n    iex> StdResult.map_or({:error, \"bar\"}, 42, &String.length/1)\n    42","ref":"StdResult.html#map_or/3-examples","title":"Examples - StdResult.map_or/3","type":"function"},{"doc":"Maps a Result to a value by applying fallback function `default` to a\ncontained `Err` value, or function `fun` to a contained `Ok` value.\n\nThis function can be used to unpack a successful result while handling an\nerror.","ref":"StdResult.html#map_or_else/3","title":"StdResult.map_or_else/3","type":"function"},{"doc":"iex> StdResult.map_or_else({:ok, \"foo\"}, &(&1), &String.length/1)\n    3\n\n    iex> StdResult.map_or_else({:error, \"bar\"}, &(&1), &String.length/1)\n    \"bar\"","ref":"StdResult.html#map_or_else/3-examples","title":"Examples - StdResult.map_or_else/3","type":"function"},{"doc":"Wrap the input inside a tuple if needed.\n\nNOTE: This function does not try to guess the type of Result and needs\nto receive an atom `:ok` or `:error`. If you want to wrap any term, you\ncan use `ok/1` or `err/1`.\n\nIt is mainly used to manipulate, normalize and chain Elixir functions\nsuch as `System.fetch_env/1` or `System.delete_env/1` which return\n`:ok` or `:error` without tuple.","ref":"StdResult.html#normalize_result/1","title":"StdResult.normalize_result/1","type":"function"},{"doc":"iex> StdResult.normalize_result(:ok)\n    {:ok, {}}\n\n    iex> StdResult.normalize_result(:error)\n    {:error, {}}\n\n    iex> StdResult.normalize_result({:ok, :foo})\n    {:ok, :foo}\n\n    iex> StdResult.normalize_result({:error, :bar})\n    {:error, :bar}","ref":"StdResult.html#normalize_result/1-examples","title":"Examples - StdResult.normalize_result/1","type":"function"},{"doc":"Wrap any term inside a `:ok` tuple.","ref":"StdResult.html#ok/1","title":"StdResult.ok/1","type":"function"},{"doc":"iex> StdResult.ok(\"value\")\n    {:ok, \"value\"}","ref":"StdResult.html#ok/1-examples","title":"Examples - StdResult.ok/1","type":"function"},{"doc":"Same as `ok/1` but can be used in guards, pattern matching, ...","ref":"StdResult.html#ok!/1","title":"StdResult.ok!/1","type":"macro"},{"doc":"Returns `true` if the result is `Ok`.","ref":"StdResult.html#ok?/1","title":"StdResult.ok?/1","type":"function"},{"doc":"iex> StdResult.ok?({:ok, -3})\n    true\n\n    iex> StdResult.ok?({:error, \"Some error message\"})\n    false","ref":"StdResult.html#ok?/1-examples","title":"Examples - StdResult.ok?/1","type":"function"},{"doc":"Returns `true` if the result is `Ok` and the value inside of\nit matches a predicate.","ref":"StdResult.html#ok_and?/2","title":"StdResult.ok_and?/2","type":"function"},{"doc":"iex> StdResult.ok_and?({:ok, 2}, &(&1 > 1))\n    true\n\n    iex> StdResult.ok_and?({:ok, 0}, &(&1 > 1))\n    false\n\n    iex> StdResult.ok_and?({:error, \"hey\"}, &(&1 > 1))\n    false","ref":"StdResult.html#ok_and?/2-examples","title":"Examples - StdResult.ok_and?/2","type":"function"},{"doc":"Calls the callback if the result is `Err`, otherwise returns the `Ok` value.\n\nThis function can be used for control flow based on Result values.","ref":"StdResult.html#or_else/2","title":"StdResult.or_else/2","type":"function"},{"doc":"iex> StdResult.or_else({:ok, 2}, &{:ok, &1 * 2})\n    {:ok, 2}\n\n    iex> StdResult.or_else({:ok, 2}, &{:error, &1 * 4})\n    {:ok, 2}\n\n    iex> StdResult.or_else({:error, 2}, &{:ok, &1 * 2})\n    {:ok, 4}\n\n    iex> StdResult.or_else({:error, 2}, &{:error, &1 * 4})\n    {:error, 8}","ref":"StdResult.html#or_else/2-examples","title":"Examples - StdResult.or_else/2","type":"function"},{"doc":"Returns the right operand if the result is `Err`, otherwise returns the\n`Ok` value.\n\nArguments passed to `or_result/2` are eagerly evaluated; if you are\npassing the result of a function call, it is recommended to use\n`or_else/2`, which is lazily evaluated.","ref":"StdResult.html#or_result/2","title":"StdResult.or_result/2","type":"function"},{"doc":"iex> StdResult.or_result({:ok, 2}, {:error, \"late error\"})\n    {:ok, 2}\n\n    iex> StdResult.or_result({:error, \"early error\"}, {:ok, 2})\n    {:ok, 2}\n\n    iex> StdResult.or_result({:error, \"not a 2\"}, {:error, \"late error\"})\n    {:error, \"late error\"}\n\n    iex> StdResult.or_result({:ok, 2}, {:ok, 100})\n    {:ok, 2}","ref":"StdResult.html#or_result/2-examples","title":"Examples - StdResult.or_result/2","type":"function"},{"doc":"Partition a sequence of Results into one list of all the `Ok` elements\nand another list of all the `Err` elements.","ref":"StdResult.html#partition_result/1","title":"StdResult.partition_result/1","type":"function"},{"doc":"iex> ok_and_errors = [{:ok, 1}, {:error, false}, {:error, true}, {:ok, 2}]\n    iex> StdResult.partition_result(ok_and_errors)\n    {[1, 2], [false, true]}","ref":"StdResult.html#partition_result/1-examples","title":"Examples - StdResult.partition_result/1","type":"function"},{"doc":"The `{}` type, also called \"unit\".\n\nThe `{}` type has exactly one value `{}`, and is used when there is no\nother meaningful value that could be returned.","ref":"StdResult.html#unit/0","title":"StdResult.unit/0","type":"function"},{"doc":"iex> StdResult.unit()\n    {}","ref":"StdResult.html#unit/0-examples","title":"Examples - StdResult.unit/0","type":"function"},{"doc":"Same as `unit/0` but can be used in guards, pattern matching, ...","ref":"StdResult.html#unit!/0","title":"StdResult.unit!/0","type":"macro"},{"doc":"Returns the contained `Ok` value.\n\nBecause this function may raise, its use is generally discouraged.\nInstead, prefer to use pattern matching and handle the `Err` case\nexplicitly, or call `unwrap_or/2` or `unwrap_or_else/2`.","ref":"StdResult.html#unwrap/1","title":"StdResult.unwrap/1","type":"function"},{"doc":"iex> StdResult.unwrap({:ok, 21})\n    21\n\n    iex> StdResult.unwrap({:error, \"emergency failure\"})\n    ** (RuntimeError) emergency failure","ref":"StdResult.html#unwrap/1-examples","title":"Examples - StdResult.unwrap/1","type":"function"},{"doc":"Returns the contained `Err` value.\n\nFor more details, see: `unwrap/1`","ref":"StdResult.html#unwrap_err/1","title":"StdResult.unwrap_err/1","type":"function"},{"doc":"iex> StdResult.unwrap_err({:error, \"emergency failure\"})\n    \"emergency failure\"\n\n    iex> StdResult.unwrap_err({:ok, 42})\n    ** (RuntimeError) 42","ref":"StdResult.html#unwrap_err/1-examples","title":"Examples - StdResult.unwrap_err/1","type":"function"},{"doc":"Returns the contained `Ok` value or a provided default.\n\nArguments passed to `unwrap_or/2` are eagerly evaluated; if you are\npassing the result of a function call, it is recommended to use\n`unwrap_or_else/2`, which is lazily evaluated.","ref":"StdResult.html#unwrap_or/2","title":"StdResult.unwrap_or/2","type":"function"},{"doc":"iex> StdResult.unwrap_or({:ok, 2}, 42)\n    2\n\n    iex> StdResult.unwrap_or({:error, \"error\"}, 42)\n    42","ref":"StdResult.html#unwrap_or/2-examples","title":"Examples - StdResult.unwrap_or/2","type":"function"},{"doc":"Returns the contained `Ok` value or computes it from a closure.","ref":"StdResult.html#unwrap_or_else/2","title":"StdResult.unwrap_or_else/2","type":"function"},{"doc":"iex> StdResult.unwrap_or_else({:ok, 2}, &String.length/1)\n    2\n\n    iex> StdResult.unwrap_or_else({:error, \"foo\"}, &String.length/1)\n    3","ref":"StdResult.html#unwrap_or_else/2-examples","title":"Examples - StdResult.unwrap_or_else/2","type":"function"},{"doc":"","ref":"StdResult.html#t:err/0","title":"StdResult.err/0","type":"type"},{"doc":"","ref":"StdResult.html#t:err/1","title":"StdResult.err/1","type":"type"},{"doc":"","ref":"StdResult.html#t:ok/0","title":"StdResult.ok/0","type":"type"},{"doc":"","ref":"StdResult.html#t:ok/1","title":"StdResult.ok/1","type":"type"},{"doc":"","ref":"StdResult.html#t:result/0","title":"StdResult.result/0","type":"type"},{"doc":"","ref":"StdResult.html#t:result/2","title":"StdResult.result/2","type":"type"}]}